﻿using OntimeScannerAPI;
using OntimeScannerAPI.API;
using OntimeScannerAPI.Correspondence;
using OntimeScannerAPI.Data;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace OntimeScannerDemo
{
	public partial class Main : Form
	{

		List<string> epcs = new List<string>();

		public Main()
		{
			InitializeComponent();

			MaximizeBox = false;//使最大化窗口失效           
			FormBorderStyle = FormBorderStyle.FixedSingle; //禁止对窗口大小进行拖拽   
			StartPosition = FormStartPosition.CenterScreen;  //设置属性窗口居中

			GetComs();


			InitReaderEvent();
		}


		private void InitReaderEvent()
		{
			ReaderApi.Instance().InventoryTagEventHandler += OnTagData;
			ReaderApi.Instance().OnSettingEventHandler += OnSetting;
			ReaderApi.Instance().HeartbeatEventHandlerh += OnHandlerh;
			ReaderApi.Instance().OnKeypadEventHandler += OnKeypad;
		}

		private void GetComs()
		{
			comboBoxCom.Text = "";
			comboBoxCom.Items.Clear();

			string[] str = SerialPort.GetPortNames();
			if (str == null)
			{
				MessageBox.Show("本机没有串口！", "Error");
				return;
			}

			//添加串口
			foreach (string s in str)
			{
				comboBoxCom.Items.Add(s);
			}
		}

		private void BtnConn_Click(object sender, EventArgs e)
		{

			if (string.IsNullOrEmpty(comboBoxCom.Text))
			{

				MessageBox.Show("请和选择有效串口");
				return;
			}

			if (BtnConn.Text.Equals("连接"))
			{

				if (ReaderApi.Instance().Connect(comboBoxCom.Text))
				{

					MessageBox.Show("连接成功");
					BtnConn.Text = "断开";
					//连接设备后设置工作模式为RFID,否则无法读取标签和配置功率
					ReaderApi.Instance().SetWorkingMode(WorkingMode.RFID);
				}
				else
				{
					MessageBox.Show("连接失败");
				}
			}
			else {


				ReaderApi.Instance().Close();
				BtnConn.Text = "连接";
			}

		
		}

		private void BtSet_Click(object sender, EventArgs e)
		{
			if (string.IsNullOrEmpty(cmbPower.Text))
			{

				MessageBox.Show("功率不能为空");
				return;
			}

			ReaderApi.Instance().SetPower(Convert.ToInt32(cmbPower.Text));
		}

		/// <summary>
		/// 获取设备读取数据
		/// </summary>
		/// <param name="tag"></param>
		private void OnTagData(string tag)
		{
			Console.WriteLine("EPC:" + tag);

			Invoke((EventHandler)(delegate
			{
				Inventory(tag);
			}));

		}

		//每隔几秒自动返回电量值
		private void OnHandlerh(int v)
		{
			Invoke((EventHandler)(delegate
			{
				lbBattery.Text = v * 10 + "%";
				
			}));
		}

		/// <summary>
		/// 监听开始/暂停按钮反馈
		/// </summary>
		/// <param name="keypad"></param>
		private void OnKeypad(KeypadStatus keypad)
		{
			Invoke((EventHandler)(delegate
			{
				BtStart_Click(null,null);
			}));
		}

		private void OnSetting(Setting setting)
		{
			Invoke((EventHandler)(delegate
			{
				listBoxLog.Items.Clear();

				listBoxLog.Items.Add("获取功率值：" + setting.power);
				if (setting.sound){

					rbSundOpen.Checked = true;
				}
				else {
					rbSundOff.Checked = true;
				}
				listBoxLog.Items.Add("获取声音模式：" + (setting.sound ? "开" : "关"));
			}));
		}

		private void BtClean_Click(object sender, EventArgs e)
		{
			lvEpc.Items.Clear();
			lbCount.Text = (lvEpc.Items.Count).ToString();
			epcs.Clear();
		}

		private void BtStart_Click(object sender, EventArgs e)
		{
			if (BtStart.Text.Equals("开始盘询"))
			{
				ReaderApi.Instance().StartInventory();
				BtStart.Text = "停止盘询";
				BtSet.Enabled = false;
				gruSound.Enabled = false;
				BtGet.Enabled = false;
			}
			else {

				ReaderApi.Instance().StopInventory();
				BtStart.Text = "开始盘询";
				BtSet.Enabled = true;
				BtGet.Enabled = true;
				gruSound.Enabled = true;
			}
		}

		private void Inventory(string epc )
		{
			if (!epcs.Contains(epc))
			{
				epcs.Add(epc);
				ListViewItem li = new ListViewItem();
				li.Text = epc;
				li.SubItems.Add("1");

				lvEpc.Items.Add(li);
				lbCount.Text = (lvEpc.Items.Count).ToString();
			}
			else
			{
				int index = FindRowIndex(lvEpc, epc, 0);
				if (index == -1) return;

				string cnt = Convert.ToString(Convert.ToInt32(lvEpc.Items[index].SubItems[1].Text) + 1);
				lvEpc.Items[index].SubItems[1].Text = cnt;
			}
		}

		private int FindRowIndex(ListViewNF lv, string text, int columnIndex)
		{
			foreach (ListViewItem li in lv.Items)
			{
				if (li.SubItems[columnIndex].Text == text)
				{
					return li.Index;
				}
			}
			return -1;
		}

		private void BtSetSund_Click(object sender, EventArgs e)
		{
			ReaderApi.Instance().SetSonudMode(rbSundOpen.Checked ? true : false);
		}

		private void BtGet_Click(object sender, EventArgs e)
		{
			ReaderApi.Instance().GetSonudMode();
			ReaderApi.Instance().GetPower();
		}
	}
}
